// ----------------------
if ( typeof DEBUG === "undefined" ) DEBUG = true; //for uglifyjs

// Config
SERVER_NAME = 'http://localhost:5000/save_searches' // Development Server
ADD_AUTOCOMPLETE = false // add autocomplete suggestions to queue 

// Base URLs
var googleSearch = "https://www.google.com/search?q="
var googleAutocomplete = "https://www.google.com/complete/search?sclient=psy-ab&q="

// Search terms list
var searchTerms = [
"2017+US+President"//, "US+President",
// "inauguration", "Trump+inauguration", "President+inauguration",
// "democrat", "liberal",
// "independent", "moderate",
// "republican", "conservative",
// "Donald", "Trump", "Donald+Trump",
// "Mike", "Pence", "Mike+Pence",
// "Putin", "Russia",
// "China", "UN"
]

// Milliseconds between searches
var WAIT_TIME = 30000;

// Set search queue index
var search_index = 0;

// Set window variables
standardWinId = null
incognitoWinId = null

// ----------------------

// Date Formatting
function pad(value) {
  value = (value > 9) ? value : "0" + value;
  return String(value);
}

// Create Time Stamp
function timeStamp() {
  var currentTime = new Date();
  var datestr = currentTime.getFullYear() + "" + pad(currentTime.getMonth() + 1 ) + ""+ pad(currentTime.getDate());
  var timestr = pad(currentTime.getHours()) + "" + pad(currentTime.getMinutes()) + "" + pad(currentTime.getSeconds()) + "" + pad(currentTime.getMilliseconds());  
  return datestr + "_" + timestr;
}

// Get Autocomplete Suggestions
function getAutoComplete(terms) {
  var httpRequest = new XMLHttpRequest();
  query = googleAutocomplete + terms + "%20"
  httpRequest.open("GET", query, false);
  httpRequest.send();
  content = httpRequest.responseText
  return content
}

// Extract text content from html
function extractContent(s) {
  var span= document.createElement('span');
  span.innerHTML = s;
  return span.textContent || span.innerText;
};

// Decode unicode to char
function convertUnicode(autocomplete) {
  var r = /\\u([\d\w]{4})/gi;
  ac_terms = autocomplete.replace(r, function (match, grp) {
    return String.fromCharCode(parseInt(grp, 16)); } );
  return ac_terms
}

// Parse suggestions from `getAutoComplete`
function parseAutoComplete(autocomplete){
  ac = convertUnicode(autocomplete) //decode unicode
  ac = ac.replace(/<\\\/b>/g, '') // replace escaped html tag
  ac = extractContent(ac) // remove removing html
  ac = ac.replace(/"/g, "") // remove quotes
  ac = ac.match("\\[(.*)]")[1] // extract from within brackets
  ac = ac.match("\\[(.*)]")[1]
  ac = ac.match("\\[(.*)]")[1].split('],[')
  suggests = []
  for (i = 0; i < ac.length; i++){
    suggests[i] = ac[i].split(',')[0]
  }
  return suggests
}

// Replace space with +
function formatSearchTerms(search_terms){
  for (i = 0; i < search_terms.length; i++){
    search_terms[i] = search_terms[i].replace(' ', '+');
  }
  return search_terms
}

// Do the deed
function doSearch() {
  var j = search_index;

  if (j == 0){ // Set length on first loop
    n_terms = searchTerms.length
  }

  searchUrl = googleSearch + searchTerms[j]
  // Get autocomplete for current query
  var autocomplete = getAutoComplete(searchTerms[j]);
  DEBUG && console.log("---Autocomplete---");
  DEBUG && console.log("Autocomplete: ", autocomplete);
  // sendAutoCompleteData(userId, autocomplete);
  
  if (ADD_AUTOCOMPLETE){
    if (search_index < n_terms){ // Add autocomplete for original terms
      add_terms = parseAutoComplete(autocomplete)
      DEBUG && console.log("Parsed Autocomplete: ", add_terms);
      formatted_terms = formatSearchTerms(add_terms)
      searchTerms = searchTerms.concat(formatted_terms);
    }   
  }
  
 chrome.tabs.create({ // Create new tab for each URL
    windowId: standardWinId,
    url: searchUrl,
    active: false
  },
  function(tab) { // With newly opened tab
    DEBUG && console.log("---Standard---")
    DEBUG && console.log("Tab ID: ", tab.id);
    DEBUG && console.log("Tab URL: ", tab.url);
    var type = "standard";
    chrome.tabs.onUpdated.addListener( // Add event listener to new tab
      function func(tabId, changeInfo) { // Wait for tab to load
        if (tabId == tab.id && changeInfo.status == 'complete') {
          chrome.tabs.onUpdated.removeListener(func);
          savePage(tab, userId, autocomplete, type);
        }
      }
    );
  });

  chrome.tabs.create({ // Create new tab for each URL
    windowId: incognitoWinId,
    url: searchUrl,
    active: false
  }, 
  function(tab) { // With newly opened tab
    DEBUG && console.log("---Incognito---")
    DEBUG && console.log("Tab ID: ", tab.id);
    DEBUG && console.log("Tab URL: ", tab.url);
    var type = "incognito";
    chrome.tabs.onUpdated.addListener( // Add event listener to new tab
      function func(tabId, changeInfo) { // Wait for tab to load
        if (tabId == tab.id && changeInfo.status == 'complete') {
          chrome.tabs.onUpdated.removeListener(func);
          savePage(tab, userId, autocomplete, type);
        }
      }
    );
  });   
  
  search_index += 1;
  if (search_index < searchTerms.length) setTimeout(doSearch, WAIT_TIME);

  if (search_index == searchTerms.length){ // Display finished.html
    chrome.tabs.update(standardTabId, {url: "finished.html"});
    chrome.tabs.remove(incognitoTabId);
  }
}

// Save Webpage HTML
function savePage(tab, userId, autocomplete, type) {
  chrome.tabs.sendMessage(tab.id, 
    { // Message content.js to grab DOM from opened tab
      'From': 'background',
      'uid': userId,
      'autocomplete': autocomplete,
      'type': type
    },
    function(obj){ // With data obj returned from content.js
      sendData(obj);
      chrome.tabs.remove(tab.id);
    }
  );
}

// Send Data to Server
function sendData(obj) {
  $.ajax({
    type: "POST",
    url: SERVER_NAME,
    data: JSON.stringify({
      "domContent": obj.domContent,
      "user_id" : obj.uid,
      "time_stamp" : timeStamp(),
      "type": obj.type,
      "autocomplete": obj.autocomplete
    }),
    dataType: "json",
    'Content-Type': "application/json",
    success: function(res) {
      //alert("great success");
    },
    error: function(e) {
      console.log(e)
    }
  });
}

// After receiving ID
chrome.runtime.onMessage.addListener(
  function(request, sender, sendResponse) {
    if(request.From == "popup"){
      userId = request.Value
      DEBUG && console.log("ID Submitted in popup.js");
      DEBUG && console.log("ID: ", userId);
      
      chrome.windows.create({ // Create standard window
        focused: false
      }, function(newWindow) {
        DEBUG && console.log(newWindow)
        standardWinId = newWindow.id;
        standardTabId = newWindow.tabs[0].id
        DEBUG && console.log("Standard Window ID: ", standardWinId);
        DEBUG && console.log("Standard Window Tab: ", standardTabId);
      });

      chrome.windows.create({ // Create incognito window 
        // NOTE - User MUST check "Allow in incognito mode" in chrome://extensions 
        // for this to work. Returns null otherwise.
        focused: false,
        incognito: true
      }, function(newWindow) {
        DEBUG && console.log(newWindow)
        incognitoWinId = newWindow.id;
        incognitoTabId = newWindow.tabs[0].id
        DEBUG && console.log("Incognito Window ID: ", incognitoWinId);        
        DEBUG && console.log("Incognito Window Tab: ", incognitoTabId);
      });

    // Run searches
    setTimeout(doSearch, WAIT_TIME);
    }
  }
)
